use diesel::{Connection, sqlite::{SqliteConnection, Sqlite}};
use hmac_sha256::Hash;
use diesel_migrations::{embed_migrations, EmbeddedMigrations, MigrationHarness};

use crate::{config::CORE_CFG, logging::{LogLevel, log}, to_hex};

pub mod schema;
pub mod user;

pub const MIGRATIONS: EmbeddedMigrations = embed_migrations!("migrations");

fn run_migrations(connection: &mut impl MigrationHarness<Sqlite>) -> bool {
    match connection.run_pending_migrations(MIGRATIONS) {
        Ok(m) => {
            if m.len() > 0 {                
                log(LogLevel::Fatal, "db", "run_migrations", format!("New DB Version: {:?}", m[0]).as_str());
            }
            return true
        },
        Err(e) => {
            log(LogLevel::Fatal, "db", "run_migrations", format!("Failed to run migrations: {:?}", e).as_str());
            return false
        },
    }
}

pub fn connect() -> Option<SqliteConnection> {
    let cfg = CORE_CFG.get().unwrap();
    let t = SqliteConnection::establish(&cfg.sqlite_file);
    
    if t.is_err() {
        log(LogLevel::Fatal, "Database", "connect", format!("Failed to connect to database: {:?}", t.err()).as_str());
        return None;
    }

    return Some(t.unwrap())
}

pub fn init_db() -> bool {
    let conn = connect();
    if conn.is_none() {
        return false;
    }
    let mut conn = conn.unwrap();
    
    let test_res = conn.begin_test_transaction();
    if test_res.is_err() {
        log(LogLevel::Fatal, "Database", "init_db", format!("Test Transaction failed: {}", test_res.err().unwrap()).as_str());
        return false;
    }

    return run_migrations(&mut conn);
}

pub fn get_hashed_uid(user_id: &u64) -> String {
    if CORE_CFG.get().unwrap().hash_ids {
        let mut h: Hash = Hash::new();
        h.update(user_id.to_ne_bytes());
        let h_final: [u8; 32] = h.finalize();
        return to_hex(h_final.to_vec());
    } else {
        return user_id.to_string();
    }
}