use diesel::{prelude::*, insert_into, update};
use chrono::{NaiveDateTime, Utc};
use crate::{logging::{LogLevel, log}, db::{connect, get_hashed_uid}};

#[derive(Insertable, Selectable, Queryable, Debug, Clone, Default)]
#[diesel(table_name = crate::db::schema::guild)]
pub struct Guild {
    pub guild_id: i64,
    pub channel_id: i64,
}

#[derive(Insertable, Selectable, Queryable, Debug, Clone, Default)]
#[diesel(belongs_to(Guild))]
#[diesel(table_name = crate::db::schema::user)]
pub struct User {
    pub discord_id: String,
    pub server: Option<i64>,
    pub fake_id: i64,
    pub image_idx: i32,
    pub when_generated: NaiveDateTime,
    pub should_notify: bool,
    pub is_guild_admin: bool,
    pub is_banned: bool,
    pub suspend_expires: Option<NaiveDateTime>,
}

pub fn create_user(user_id: &u64, srv: Option<i64>, fid: i64, img_idx: i32) -> Option<User> {
    use crate::db::schema::user::dsl::*;
    let conn = connect();
    if conn.is_none() {
        return None
    }
    let mut conn = conn.unwrap();
    let uid: String = get_hashed_uid(user_id);

    let res = conn.transaction(|c| {
        let sql = insert_into(user)
            .values((
                discord_id.eq(uid),
                server.eq(srv),
                fake_id.eq(fid),
                image_idx.eq(img_idx)
            ))
            .execute(c);
        
            match sql {
                Err(e) => {
                    log(LogLevel::Error, "db", "create_user", format!("SQL Error: {:?}", e).as_str());
                    return Err(e);
                }
                Ok(usr) => {
                    return Ok(usr)
                }
            }
    });
    
    match res {
        Ok(_) => return get_user_by_id(user_id),
        Err(_) => return None,
    }
}

pub fn get_user_by_id(user_id: &u64) -> Option<User> {
    use crate::db::schema::user::dsl::*;
    let conn = connect();
    if conn.is_none() {
        return None
    }
    let mut conn = conn.unwrap();
    let uid: String = get_hashed_uid(user_id);

    let res = conn.transaction(|c| {
        let sql = user
            .filter(discord_id.eq(uid))
            .select(User::as_select())
            .load::<User>(c);

        match sql {
            Err(e) => {
                log(LogLevel::Error, "db", "get_user_by_id", format!("SQL Error: {:?}", e).as_str());
                return Err(e);
            }
            Ok(usr) => {
                if usr.len() > 0 {
                    return Ok(Some(usr[0].clone()))
                }
                return Ok(None)
            }
        }
    });

    match res {
        Ok(u) => return u,
        Err(_) => return None,
    }
}

pub fn set_fake_id(user_id: &u64, new_fake_id: i64, new_pic_idx: i32) {
    use crate::db::schema::user::dsl::*;
    let conn = connect();
    if conn.is_none() {
        return
    }
    let mut conn = conn.unwrap();
    let uid: String = get_hashed_uid(user_id);
    let now: NaiveDateTime = Utc::now().naive_utc();

    let _ = conn.transaction(|c| {
        let sql = update(user)
        .filter(discord_id.eq(uid))
        .set((fake_id.eq(new_fake_id), when_generated.eq(now), image_idx.eq(new_pic_idx)))
        .execute(c);
        
        match sql {
            Err(e) => {
                log(LogLevel::Error, "db", "set_fake_id", format!("SQL Error: {:?}", e).as_str());
                return Err(e);
            }
            Ok(_) => {
                return Ok(());
            }
        }
    });
}

pub fn get_user_by_fake_id(user_id: &u64) -> Option<User> {
    use crate::db::schema::user::dsl::*;
    let conn = connect();
    if conn.is_none() {
        return None
    }
    let mut conn = conn.unwrap();

    let res = conn.transaction(|c| {
        let sql = user.filter(fake_id.eq(*user_id as i64))
            .select(User::as_select())
            .load::<User>(c);
        
            match sql {
                Err(e) => {
                    log(LogLevel::Error, "db", "get_user_by_id", format!("SQL Error: {:?}", e).as_str());
                    return Err(e);
                }
                Ok(usr) => {
                    if usr.len() > 0 {
                        return Ok(Some(usr[0].clone()))
                    }
                    return Ok(None)
                }
            }
        });
    
        match res {
            Ok(u) => return u,
            Err(_) => return None,
        }
}

pub fn set_notify(user_id: &u64, is_notify: bool) {
    use crate::db::schema::user::dsl::*;
    let conn = connect();
    if conn.is_none() {
        return
    }
    let mut conn = conn.unwrap();
    let uid: String = get_hashed_uid(user_id);

    let _ = conn.transaction(|c| {
        let sql = update(user)
        .filter(discord_id.eq(uid))
        .set(should_notify.eq(is_notify))
        .execute(c);
        
        match sql {
            Err(e) => {
                log(LogLevel::Error, "db", "set_notify", format!("SQL Error: {:?}", e).as_str());
                return Err(e);
            }
            Ok(_) => {
                return Ok(());
            }
        }
    });
}