use once_cell::sync::OnceCell;
use std::sync::atomic::*;
use std::fs::{OpenOptions, File, create_dir_all, copy};
use std::io::Write;

#[derive(Debug, PartialEq, PartialOrd)]
pub enum LogLevel {
    Trace = 5,
    Debug = 10,
    Info = 20,
    Warn = 30,
    Error = 40,
    Critical = 50,
    Fatal = 100,
}

impl std::fmt::Display for LogLevel {
    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
        match *self {
            LogLevel::Trace => write!(f, "T"),
            LogLevel::Debug => write!(f, "D"),
            LogLevel::Info => write!(f, "I"),
            LogLevel::Warn => write!(f, "W"),
            LogLevel::Error => write!(f, "E"),
            LogLevel::Critical => write!(f, "C"),
            LogLevel::Fatal => write!(f, "F"),
        }
    }
}

impl std::str::FromStr for LogLevel {
    type Err = ();

    fn from_str(input: &str) -> Result<LogLevel, Self::Err> {
        match input {
            "trace" => Ok(LogLevel::Trace),
            "debug" => Ok(LogLevel::Debug),
            "info" => Ok(LogLevel::Info),
            "warn" => Ok(LogLevel::Warn),
            "warning" => Ok(LogLevel::Warn),
            "error" => Ok(LogLevel::Error),
            "critical" => Ok(LogLevel::Critical),
            "fatal" => Ok(LogLevel::Fatal),
            _ => Err(()),
        }
    }
}

static LEVEL: OnceCell<LogLevel> = OnceCell::new();
static LOG_FILE: OnceCell<File> = OnceCell::new();
static HAS_INITED: AtomicBool = AtomicBool::new(false);
static FILE_IS_GOOD: AtomicBool = AtomicBool::new(false);
static FILE_DATE: OnceCell<String> = OnceCell::new();
static FILE_FOLDER: OnceCell<String> = OnceCell::new();

pub fn init_logger(log_folder: &str, level: LogLevel) {
    if HAS_INITED.load(Ordering::Relaxed) {
        return;
    }
    LEVEL.set(level).unwrap();
    
    HAS_INITED.store(true, Ordering::Relaxed);

    let _ = FILE_DATE.set(chrono::offset::Local::now().format("%F").to_string());

    let log_file = format!("{}/whisper.log", log_folder);
    let _ = FILE_FOLDER.set(log_folder.to_string());
    let dir_create_result: Result<(), std::io::Error> = create_dir_all(log_folder);

    if dir_create_result.is_err() {
        log(LogLevel::Error, "logging", "init", format!("Error creating log folder {}", log_folder).as_str());
        return;
    }
    
    let f_ = OpenOptions::new()
        .write(true)
        .append(true)
        .create(true)
        .open(log_file);
    
    
    if f_.is_ok() {
        LOG_FILE.set(f_.unwrap()).unwrap();
        FILE_IS_GOOD.store(true, Ordering::Relaxed);
    } else {
        log(LogLevel::Error, "logging", "init", "Error getting logfile");
    }
}

pub fn log(level: LogLevel, module: &str, submodule: &str, message: &str) {
    if !HAS_INITED.load(Ordering::Relaxed) {
        println!("Logging before logger initialized!");
        return;
    }
    let timestamp: String = chrono::offset::Local::now().format("%F %X%.3f").to_string();
    let logstr: String = format!(
        "[{}]:{}:{}:{}:{}",
        timestamp, level, module, submodule, message
    );

    let new_time = chrono::offset::Local::now().format("%F").to_string();

    if !FILE_DATE.get().unwrap().eq(&new_time) {
        let _ = FILE_DATE.set(new_time.clone());
        let log_folder = FILE_FOLDER.get().unwrap();
        let log_file = format!("{}/whisper.log", log_folder);
        
        match copy(log_file.clone(), format!("{}/whisper-{}.log", log_folder, new_time)) {
            Err(e) => println!("Failed to copy over log! {:?}", e),
            _ => {
                match File::create(log_file) {
                    Err(e) => println!("Failed to open log file! {}", e),
                    Ok(f) => {
                        let _ = LOG_FILE.set(f);
                    }
                }
            }
        }
    }

    if level >= *LEVEL.get().unwrap() {
        println!("{logstr}");
            
        if FILE_IS_GOOD.load(Ordering::Relaxed) {
            let mut f = LOG_FILE.get().unwrap();
            let _ = writeln!(f, "{}", logstr);
        }
    }
}